# Synthetic Events

Synthetic events subsystem is using its own two-phase event dispatching algorithm. With custom event dispatching
algorithm it is possible to create new events like "click outside", gesture and DnD events.

## Event Handler

Event Handler is an object that contains information about `EventDispatcher` that will be used for an event handler
and function that will be executed when dispatcher fires an event for a target that registered an event handler.

`ivi-events` module provides a collection of event handler factories for all native events. Event handler function
factories are using camel case for their names, and all names are starting with "on" prefix.

```ts
import * as Events from "ivi-events";

const click = Events.onClick((ev) => {
  console.log("clicked");
});

const keyDown = Events.onKeyDown((ev) => {
  console.log("Key Down");
});
```

### Event Handler registration

Event Handlers are declaratively registered with a Virtual DOM API. Virtual DOM nodes that represent HTML and SVG
elements can have event handlers assigned with `e()` method.

```ts
interface VNode {
  e(events: Array<EventHandler | null> | EventHandler | null): this;
  // ...
}
```

```ts
import { Component, component, render } from "ivi";
import * as Events from "ivi-events";
import * as h from "ivi-html";

const StatefulComponent = component(class extends Component {
  private counter = 0;

  private onClick = Events.onClick((ev) => {
    this.counter++;
    this.invalidate();
  });

  render() {
    return h.div()
      // There are no restrictions in number of attached event handlers with the same type, it is possible to attach
      // multiple `onClick` event handlers.
      .e([
        this.onClick,
        this.onClick,
      ])
      .c(`Clicks: ${this.counter}`);
  }
});
```
